
constant	: expr {
				$$ = $1;
			}
			;

immed		: '$' constant {
				$$ = $2;
			}
			;
extend		: '*' constant {
				$$ = $2;
			}
			;
direct		: '@' constant {
				$$ = $2;
				if (pass == 1 && ($$ & ~0xFF))
					yyerror("direct out of range");
			}
index		: '*' X {
				$$ = 0;
			}
			| X '[' constant ']' {
				$$ = $3;
				if (pass == 1 && ((int)$$ < -126 || (int)$$ > 126))
					yyerror("offset out of range");
			}
			;

branch		: BR {
				$$ = 0x20;
			}
			| NBR {
				$$ = 0x21;
			}
			| BNC {
				$$ = 0x24;
			}
			| BC {
				$$ = 0x25;
			}
			| BZ {
				$$ = 0x27;
			}
			| BGE {
				$$ = 0x2C;
			}
			| BGT {
				$$ = 0x2E;
			}
			| BHI {
				$$ = 0x22;
			}
			| BLE {
				$$ = 0x2F;
			}
			| BLS {
				$$ = 0x23;
			}
			| BLT {
				$$ = 0x2D;
			}
			| BMI {
				$$ = 0x2B;
			}
			| BNZ {
				$$ = 0x26;
			}
			| BVC {
				$$ = 0x28;
			}
			| BVS {
				$$ = 0x29;
			}
			| BPL {
				$$ = 0x2A;
			}
			| BSR {
				$$ = 0x8D;
			}
			;

instruction	: ADD A ',' immed {
				opb(0x8B, $4);
			}
			| ADD A ',' direct {
				opb(0x9B, $4);
			}
			| ADD A ',' index {
				opb(0xAB, $4);
			}
			| ADD A ',' extend {
				opw(0xBB, $4);
			}
			| ADD B ',' immed {
				opb(0xCB, $4);
			}
			| ADD B ',' direct {
				opb(0xDB, $4);
			}
			| ADD B ',' index {
				opb(0xEB, $4);
			}
			| ADD B ',' extend {
				opw(0xFB, $4);
			}

			| ADD D ',' immed {
				opw(0xC3, $4);
			}
			| ADD D ',' direct {
				opb(0xD3, $4);
			}
			| ADD D ',' index {
				opb(0xE3, $4);
			}
			| ADD D ',' extend {
				opw(0xF3, $4);
			}

			| ADD A ',' B {
				op(0x1B);
			}

			| ADC A ',' immed {
				opb(0x89, $4);
			}
			| ADC A ',' direct {
				opb(0x99, $4);
			}
			| ADC A ',' index {
				opb(0xA9, $4);
			}
			| ADC A ',' extend {
				opw(0xB9, $4);
			}
			| ADC B ',' immed {
				opb(0xC9, $4);
			}
			| ADC B ',' direct {
				opb(0xD9, $4);
			}
			| ADC B ',' index {
				opb(0xE9, $4);
			}
			| ADC B ',' extend {
				opw(0xF9, $4);
			}

			| AND A ',' immed {
				opb(0x84, $4);
			}
			| AND A ',' direct {
				opb(0x94, $4);
			}
			| AND A ',' index {
				opb(0xA4, $4);
			}
			| AND A ',' extend {
				opw(0xB4, $4);
			}
			| AND B ',' immed {
				opb(0xC4, $4);
			}
			| AND B ',' direct {
				opb(0xD4, $4);
			}
			| AND B ',' index {
				opb(0xE4, $4);
			}
			| AND B ',' extend {
				opw(0xF4, $4);
			}

			| BIT A ',' immed {
				opb(0x85, $4);
			}
			| BIT A ',' direct {
				opb(0x95, $4);
			}
			| BIT A ',' index {
				opb(0xA5, $4);
			}
			| BIT A ',' extend {
				opw(0xB5, $4);
			}
			| BIT B ',' immed {
				opb(0xC5, $4);
			}
			| BIT B ',' direct {
				opb(0xD5, $4);
			}
			| BIT B ',' index {
				opb(0xE5, $4);
			}
			| BIT B ',' extend {
				opw(0xF5, $4);
			}

			| CLR index {
				opb(0x6F, $2);
			}
			| CLR extend {
				opw(0x7F, $2);
			}
			| CLR A {
				op(0x4F);
			}
			| CLR B {
				op(0x5F);
			}

			| CMP A ',' immed {
				opb(0x81, $4);
			}
			| CMP A ',' direct {
				opb(0x91, $4);
			}
			| CMP A ',' index {
				opb(0xA1, $4);
			}
			| CMP A ',' extend {
				opw(0xB1, $4);
			}

			| CMP B ',' immed {
				opb(0xC1, $4);
			}
			| CMP B ',' direct {
				opb(0xD1, $4);
			}
			| CMP B ',' index {
				opb(0xE1, $4);
			}
			| CMP B ',' extend {
				opw(0xF1, $4);
			}

			| CMP A ',' B {
				op(0x11);
			}

			| CPL index {
				opb(0x63, $2);
			}
			| CPL extend {
				opw(0x73, $2);
			}
			| CPL A {
				op(0x43);
			}
			| CPL B {
				op(0x53);
			}

			| NEG index {
				opb(0x60, $2);
			}
			| NEG extend {
				opw(0x70, $2);
			}
			| NEG A {
				op(0x40);
			}
			| NEG B {
				op(0x50);
			}

			| DA A {
				op(0x19);
			}

			| DEC index {
				opb(0x6A, $2);
			}
			| DEC extend {
				opw(0x7A, $2);
			}
			| DEC A {
				op(0x4A);
			}
			| DEC B {
				op(0x5A);
			}

			| XOR A ',' immed {
				opb(0x88, $4);
			}
			| XOR A ',' direct {
				opb(0x98, $4);
			}
			| XOR A ',' index {
				opb(0xA8, $4);
			}
			| XOR A ',' extend {
				opw(0xB8, $4);
			}
			| XOR B ',' immed {
				opb(0xC8, $4);
			}
			| XOR B ',' direct {
				opb(0xD8, $4);
			}
			| XOR B ',' index {
				opb(0xE8, $4);
			}
			| XOR B ',' extend {
				opw(0xF8, $4);
			}

			| INC index {
				opb(0x6C, $2);
			}
			| INC extend {
				opw(0x7C, $2);
			}
			| INC A {
				op(0x4C);
			}
			| INC B {
				op(0x5C);
			}

			| LD A ',' immed {
				opb(0x86, $4);
			}
			| LD A ',' direct {
				opb(0x96, $4);
			}
			| LD A ',' index {
				opb(0xA6, $4);
			}
			| LD A ',' extend {
				opw(0xB6, $4);
			}
			| LD B ',' immed {
				opb(0xC6, $4);
			}
			| LD B ',' direct {
				opb(0xD6, $4);
			}
			| LD B ',' index {
				opb(0xE6, $4);
			}
			| LD B ',' extend {
				opw(0xF6, $4);
			}

			| LD D ',' immed {
				opw(0xCC, $4);
			}
			| LD D ',' direct {
				opb(0xDC, $4);
			}
			| LD D ',' index {
				opb(0xEC, $4);
			}
			| LD D ',' extend {
				opw(0xFC, $4);
			}

			| MUL {
				opb(0x3D);
			}

			| OR A ',' immed {
				opb(0x8A, $4);
			}
			| OR A ',' direct {
				opb(0x9A, $4);
			}
			| OR A ',' index {
				opb(0xAA, $4);
			}
			| OR A ',' extend {
				opw(0xBA, $4);
			}
			| OR B ',' immed {
				opb(0xCA, $4);
			}
			| OR B ',' direct {
				opb(0xDA, $4);
			}
			| OR B ',' index {
				opb(0xEA, $4);
			}
			| OR B ',' extend {
				opw(0xFA, $4);
			}

			| PUSH A {
				op(0x36);
			}
			| PUSH B {
				op(0x37);
			}
			| POP A {
				op(0x32);
			}
			| POP B {
				op(0x33);
			}

			| RLC index {
				opb(0x69, $2);
			}
			| RLC extend {
				opw(0x79, $2);
			}
			| RLC A {
				op(0x49);
			}
			| RLC B {
				op(0x59);
			}

			| RRC index {
				opb(0x66, $2);
			}
			| RRC extend {
				opw(0x76, $2);
			}
			| RRC A {
				op(0x46);
			}
			| RRC B {
				op(0x56);
			}

			| ASL index {
				opb(0x68, $2);
			}
			| ASL extend {
				opw(0x78, $2);
			}
			| ASL A {
				op(0x48);
			}
			| ASL B {
				op(0x58);
			}

			| ASL D {
				op(0x05);
			}

			| ASR index {
				opb(0x67, $2);
			}
			| ASR extend {
				opw(0x77, $2);
			}
			| ASR A {
				op(0x47);
			}
			| ASR B {
				op(0x57);
			}

			| LSR index {
				opb(0x64, $2);
			}
			| LSR extend {
				opw(0x74, $2);
			}
			| LSR A {
				op(0x44);
			}
			| LSR B {
				op(0x54);
			}

			| LSR D {
				op(0x04);
			}

			| ST A ',' direct {
				opb(0x97, $4);
			}
			| ST A ',' index {
				opb(0xA7, $4);
			}
			| ST A ',' extend {
				opw(0xB7, $4);
			}
			| ST B ',' direct {
				opb(0xD7, $4);
			}
			| ST B ',' index {
				opb(0xE7, $4);
			}
			| ST B ',' extend {
				opw(0xF7, $4);
			}

			| ST D ',' direct {
				opb(0xDD, $4);
			}
			| ST D ',' index {
				opb(0xED, $4);
			}
			| ST D ',' extend {
				opw(0xFD, $4);
			}

			| SUB A ',' immed {
				opb(0x80, $4);
			}
			| SUB A ',' direct {
				opb(0x90, $4);
			}
			| SUB A ',' index {
				opb(0xA0, $4);
			}
			| SUB A ',' extend {
				opw(0xB0, $4);
			}
			| SUB B ',' immed {
				opb(0xC0, $4);
			}
			| SUB B ',' direct {
				opb(0xD0, $4);
			}
			| SUB B ',' index {
				opb(0xE0, $4);
			}
			| SUB B ',' extend {
				opw(0xF0, $4);
			}

			| SUB D ',' immed {
				opw(0x83, $4);
			}
			| SUB D ',' direct {
				opb(0x93, $4);
			}
			| SUB D ',' index {
				opb(0xA3, $4);
			}
			| SUB D ',' extend {
				opw(0xB3, $4);
			}

			| SUB A ',' B {
				op(0x10);
			}

			| SBC A ',' immed {
				opb(0x82, $4);
			}
			| SBC A ',' direct {
				opb(0x92, $4);
			}
			| SBC A ',' index {
				opb(0xA2, $4);
			}
			| SBC A ',' extend {
				opw(0xB2, $4);
			}
			| SBC B ',' immed {
				opb(0xC2, $4);
			}
			| SBC B ',' direct {
				opb(0xD2, $4);
			}
			| SBC B ',' index {
				opb(0xE2, $4);
			}
			| SBC B ',' extend {
				opw(0xF2, $4);
			}

			| LD B ',' A {
				op(0x16);
			}
			| LD A ',' B {
				op(0x17);
			}

			| TST index {
				opb(0x6D, $2);
			}
			| TST extend {
				opw(0x7D, $2);
			}
			| TST A {
				op(0x4D);
			}
			| TST B {
				op(0x5D);
			}

			| AND direct ',' immed {
				op3(0x71, $4, $2);
			}
			| AND index ',' immed {
				op3(0x61, $4, $2);
			}

			| OR direct ',' immed {
				op3(0x72, $4, $2);
			}
			| OR index ',' immed {
				op3(0x62, $4, $2);
			}

			| XOR direct ',' immed {
				op3(0x75, $4, $2);
			}
			| XOR index ',' immed {
				op3(0x65, $4, $2);
			}

			| BIT direct ',' immed {
				op3(0x7B, $4, $2);
			}
			| BIT index ',' immed {
				op3(0x6B, $4, $2);
			}

			| CMP X ',' immed {
				opw(0x8C, $4);
			}
			| CMP X ',' direct {
				opb(0x9C, $4);
			}
			| CMP X ',' index {
				opb(0xAC, $4);
			}
			| CMP X ',' extend {
				opw(0xBC, $4);
			}

			| DEC X {
				op(0x09);
			}
			| DEC SP {
				op(0x34);
			}
			| INC X {
				op(0x08);
			}
			| INC SP {
				op(0x31);
			}

			| LD X ',' immed {
				opw(0xCE, $4);
			}
			| LD X ',' direct {
				opb(0xDE, $4);
			}
			| LD X ',' index {
				opb(0xEE, $4);
			}
			| LD X ',' extend {
				opw(0xFE, $4);
			}

			| LD SP ',' immed {
				opw(0x8E, $4);
			}
			| LD SP ',' direct {
				opb(0x9E, $4);
			}
			| LD SP ',' index {
				opb(0xAE, $4);
			}
			| LD SP ',' extend {
				opw(0xBE, $4);
			}

			| ST X ',' direct {
				opb(0xDF, $4);
			}
			| ST X ',' index {
				opb(0xEF, $4);
			}
			| ST X ',' extend {
				opw(0xFF, $4);
			}

			| ST SP ',' direct {
				opb(0x9F, $4);
			}
			| ST SP ',' index {
				opb(0xAF, $4);
			}
			| ST SP ',' extend {
				opw(0xBF, $4);
			}

			| TXS {
				op(0x35);
			}
			| TSX {
				op(0x30);
			}

			| ADD X ',' B {
				op(0x3A);
			}

			| PUSH X {
				op(0x3C);
			}
			| POP X {
				op(0x38);
			}

			| XCH D ',' X {
				op(0x18);
			}
			| XCH X ',' D {
				op(0x18);
			}

			| branch constant {
				int off = $2 - dot - 2;
				if (pass == 1 && (off < -125 || off > 125))
					yyerror("relative out of range");
				opb($1, off);
			}

			| JMP index {
				opb(0x6E, $2);
			}
			| JMP constant {
				opw(0x7E, $2);
			}
			| JSR direct {
				opb(0x9D, $2);
			}
			| JSR index {
				opb(0xAD, $2);
			}
			| JSR constant {
				opw(0xBD, $2);
			}

			| NOP {
				op(0x01);
			}

			| RTI {
				op(0x3B);
			}
			| RTS {
				op(0x39);
			}

			| SWI {
				op(0x3F);
			}
			| WAI {
				op(0x3E);
			}
			| SLP {
				op(0x1A);
			}

			| CLC {
				op(0x0C);
			}
			| CLI {
				op(0x0E);
			}
			| CLV {
				op(0x0A);
			}
			| SEC {
				op(0x0D);
			}
			| SEI {
				op(0x0F);
			}
			| SEV {
				op(0x0B);
			}
			| TAP {
				op(0x06);
			}
			| TPA {
				op(0x07);
			}

			;

%%

char *cpu_family = "HD6303Y";

opw(op, wrd) { op3(op, wrd >> 8, wrd); }

